﻿using Steema.TeeChart.WPF.Functions;
using Steema.TeeChart.WPF.Styles;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;

namespace TeeChartHistogram
{
  /// <summary>
  /// Interaction logic for MainWindow.xaml
  /// </summary>
  public partial class MainWindow : Window
  {

    public struct Series
    {
      public List<double?> XValues { get; set; }
      public List<double?> YValues { get; set; }

      public void Add(double? x, double? y)
      {
        XValues.Add(x);
        YValues.Add(y);
      }

      public static Series NewSeries()
      {
        Series series = new Series
        {
          XValues = new List<double?>(),
          YValues = new List<double?>()
        };
        return series;
      }
    }

    public MainWindow()
    {
      InitializeComponent();

      Series series = Series.NewSeries();
      AddValuesToFastLine(series);

      Stopwatch stopWatch = new Stopwatch();
      stopWatch.Start();

      Series series2 = CalculateHistogram(series, 100, true, false);

      ConvertYToPercent(series2);

      Points series25 = new Points(MyTChart.Chart);
      series25.Marks.Visible = false;
      series25.XValues.Count = series2.XValues.Count;
      series25.XValues.Value = series2.XValues.Select(x => x.GetValueOrDefault()).ToArray();
      series25.YValues.Count = series2.YValues.Count;
      series25.YValues.Value = series2.YValues.Select(x => x.GetValueOrDefault()).ToArray();
      series25.Color = Colors.Red;
      series25.Title = "Test";
      series25.Visible = true;

      FastLine series3 = new FastLine(MyTChart.Chart);
      series3.Marks.Visible = false;
      series3.XValues.Count = series2.XValues.Count;
      series3.XValues.Value = series2.XValues.Select(x => x.GetValueOrDefault()).ToArray();
      series3.YValues.Count = series2.YValues.Count;
      series3.YValues.Value = series2.YValues.Select(x => x.GetValueOrDefault()).ToArray();
      series3.Function = new Smoothing();
      series3.Color = Colors.Blue;
      series3.Title = "Test1";
      series3.Visible = true;

      stopWatch.Stop();

      lblChartGenereted.Content = $"Chart is ready time taken { stopWatch.Elapsed} ";
    }

    private Series CalculateHistogram(Series s, int numbins, bool includeNulls, bool cumulative)
    {
      double[] bins = new double[numbins];
      double[] counts = new double[numbins];

      double[] tmp;

      if(includeNulls)
      {
        tmp = s.YValues.Select(x => x.GetValueOrDefault()).ToArray();
      }
      else 
      {
        tmp = s.YValues.Where(x => x.HasValue).Select(x => x.Value).ToArray();
      }

      int N = tmp.Length;
      HistogramFunction.Histogram(tmp, 0, N - 1, ref bins, ref counts, numbins, tmp.Min(), tmp.Max());

      if (cumulative)
      {
        for (int i = 1; i < numbins; i++)
        {
          counts[i] += counts[i - 1];
        }
      }

      Series series = Series.NewSeries();
      series.XValues.AddRange(bins.Select(x => (double?)x));
      series.YValues.AddRange(counts.Select(y => (double?)y));

      return series;
    }

    /// <summary>
    /// Adds the values to fast line.
    /// </summary>
    /// <param name="plotObject">The plot object.</param>
    /// <param name="series">The series.</param>
    private static void AddValuesToFastLine(Series series)
    {

      Task thread1 = Task.Factory.StartNew(() => AddValue(102721, series));
      Task thread2 = Task.Factory.StartNew(() => AddValue(102721, series));
      Task thread3 = Task.Factory.StartNew(() => AddValue(102721, series));

      Task.WaitAll(thread1, thread2, thread3);

    }

    private static readonly object balanceLock = new object();

    private static void AddValue(int count, Series series)
    {
      for (int i = 0; i <= count; i++)
      {
        double x1 = NextFloat(42109, 42108);
        double y1 = NextFloat(365, 364);
        lock (balanceLock)
        {
          series.Add(x1, y1);
        }
      }
    }
    private static readonly System.Random random = new System.Random();
    private static double NextFloat(float min, float max)
    {
      double val = (random.NextDouble() * (max - min) + min);
      return (float)val;
    }

    private void ConvertYToPercent(Series series2)
    {
      if (series2.YValues != null && series2.YValues.Count > 0)
      {
        double total = series2.YValues.Where(y => y.HasValue).Select(y => y.Value).Sum();
        series2.YValues = (from yValve in series2.YValues select (yValve / total) * 100).ToList();
      }
    }
  }
}
